@@ -1,20 +1,25 @@
-# SPDX-FileCopyrightText: 2021 REFITT Team
+# SPDX-FileCopyrightText: 2019-2021 REFITT Team
 # SPDX-License-Identifier: Apache-2.0
 
 """Package metadata for refitt."""
 
 
 __appname__     = 'refitt'
-__version__     = '0.18.0'
-__authors__     = ['Dan Milisavljevic, Ph.D. <dmilisav@purdue.edu>',
-                   'Niharika Sravan, Ph.D. <nsravan@purdue.edu>',
-                   'Geoffrey Lentner, M.Sc. <glentner@purdue.edu>',
-                   'Mark Linvill <mlinvill@purdue.edu>']
+__version__     = '0.19.0'
+__authors__     = ['Dan Milisavljevic <dmilisav@purdue.edu>',
+                   'Niharika Sravan <nsravan@purdue.edu>',
+                   'Geoffrey Lentner <glentner@purdue.edu>',
+                   'Mark Linvill <mlinvill@purdue.edu>',
+                   'Bhagya Subrayan <bsubraya@purdue.edu>',
+                   'Katie Weil <keweil@purdue.edu>',
+                   'Josh Church <church10@purdue.edu>',
+                   'John Banovetz <jbanovet@purdue.edu>',
+                   ]
 __developer__   = 'Geoffrey Lentner'
 __contact__     = 'glentner@purdue.edu'
 __license__     = 'Apache License 2.0'
 __website__     = 'https://github.com/refitt/refitt'
-__copyright__   = 'REFITT Team 2019'
+__copyright__   = 'REFITT Team 2019-2021'
 __description__ = 'The Recommender Engine for Intelligent Transient Tracking.'
 __keywords__    = 'astronomy science machine-learning recommendation service'
 __ascii_art__   = r"""
