@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2021 REFITT Team
+# SPDX-FileCopyrightText: 2019-2021 REFITT Team
 # SPDX-License-Identifier: Apache-2.0
 
 """Implements the common `Alert` interface for all broker clients."""
@@ -18,7 +18,11 @@ from ...database.model import ObjectType, Object, Source, ObservationType, Obser
 from ...database.interface import Session
 
 # public interface
-__all__ = ['AlertInterface', ]
+__all__ = ['AlertInterface', 'AlertError', ]
+
+
+class AlertError(Exception):
+    """Exceptions specific to problems with Alerts."""
 
 
 # JSON-dict
