@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2021 REFITT Team
+# SPDX-FileCopyrightText: 2019-2021 REFITT Team
 # SPDX-License-Identifier: Apache-2.0
 
 """Integration tests for /recommendation endpoints."""
@@ -15,19 +15,19 @@ import pytest
 
 # internal libs
 from refitt.database import config
-from refitt.database.model import Recommendation, RecommendationGroup, User, Facility, Observation, File
+from refitt.database.model import Recommendation, Epoch, User, Facility, Observation, File
 from refitt.web.api.response import (STATUS, RESPONSE_MAP, NotFound, ParameterInvalid, ParameterNotFound,
-                                     PermissionDenied, PayloadTooLarge, PayloadMalformed)
+                                     PermissionDenied, PayloadMalformed)
 from refitt.web.api.endpoint.recommendation import recommendation_slices
 from tests.integration.test_web.test_api.test_endpoint import Endpoint
 
 
 @contextmanager
-def all_unseen(user_id: int, group_id: int) -> None:
+def all_unseen(user_id: int, epoch_id: int) -> None:
     """Temporarily set `accept` and `reject` to false."""
     previous = {
         record.id: (record.accepted, record.rejected)
-        for record in Recommendation.query().filter_by(user_id=user_id, group_id=group_id).all()
+        for record in Recommendation.query().filter_by(user_id=user_id, epoch_id=epoch_id).all()
     }
     try:
         for id in previous:
@@ -58,8 +58,8 @@ class TestGetNextRecommendation(Endpoint):
     def test_all_accepted(self) -> None:
         """Should be empty if all are already accepted/rejected (as in test data)."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
         assert self.get(self.route, client_id=client.id) == (
             STATUS['OK'], {
                 'Status': 'Success',
@@ -72,10 +72,10 @@ class TestGetNextRecommendation(Endpoint):
     def test_all_unseen(self) -> None:
         """Should get all recommendations for the user if none have been accepted or rejected."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
-        with all_unseen(user_id=client.user_id, group_id=group_id):
-            recommendations = Recommendation.next(user_id=client.user_id, group_id=group_id)
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
+        with all_unseen(user_id=client.user_id, epoch_id=epoch_id):
+            recommendations = Recommendation.next(user_id=client.user_id, epoch_id=epoch_id)
             assert len(recommendations) == 4
             assert self.get(self.route, client_id=client.id) == (
                 STATUS['OK'], {
@@ -89,10 +89,10 @@ class TestGetNextRecommendation(Endpoint):
     def test_with_limit(self) -> None:
         """Should only return the first recommendation of the unseen for given group."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
-        with all_unseen(user_id=client.user_id, group_id=group_id):
-            recommendations = Recommendation.next(user_id=client.user_id, group_id=group_id, limit=1)
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
+        with all_unseen(user_id=client.user_id, epoch_id=epoch_id):
+            recommendations = Recommendation.next(user_id=client.user_id, epoch_id=epoch_id, limit=1)
             assert len(recommendations) == 1
             assert self.get(self.route, client_id=client.id, limit=1) == (
                 STATUS['OK'], {
@@ -106,14 +106,14 @@ class TestGetNextRecommendation(Endpoint):
     def test_with_facility(self) -> None:
         """Only return recommendations for the given facility."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
-        with all_unseen(user_id=client.user_id, group_id=group_id):
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
+        with all_unseen(user_id=client.user_id, epoch_id=epoch_id):
             facility = Facility.from_name('Croft_1m')
-            recommendations = Recommendation.next(user_id=client.user_id, group_id=group_id,
+            recommendations = Recommendation.next(user_id=client.user_id, epoch_id=epoch_id,
                                                   facility_id=facility.id)
             assert len(recommendations) == 2
-            assert all(recommendation.group_id == group_id for recommendation in recommendations)
+            assert all(recommendation.epoch_id == epoch_id for recommendation in recommendations)
             assert all(recommendation.facility_id == facility.id for recommendation in recommendations)
             assert self.get(self.route, client_id=client.id, facility_id=facility.id) == (
                 STATUS['OK'], {
@@ -127,14 +127,14 @@ class TestGetNextRecommendation(Endpoint):
     def test_with_facility_and_limiting_magnitude(self) -> None:
         """Only return recommendations for the given facility and limiting magnitude."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
-        with all_unseen(user_id=client.user_id, group_id=group_id):
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
+        with all_unseen(user_id=client.user_id, epoch_id=epoch_id):
             facility = Facility.from_name('Croft_1m')
-            recommendations = Recommendation.next(user_id=client.user_id, group_id=group_id,
+            recommendations = Recommendation.next(user_id=client.user_id, epoch_id=epoch_id,
                                                   facility_id=facility.id, limiting_magnitude=15.4)
             assert len(recommendations) == 1
-            assert all(recommendation.group_id == group_id for recommendation in recommendations)
+            assert all(recommendation.epoch_id == epoch_id for recommendation in recommendations)
             assert all(recommendation.facility_id == facility.id for recommendation in recommendations)
             assert recommendations[0].predicted.value <= 15.4
             assert self.get(self.route, client_id=client.id, facility_id=facility.id, limiting_magnitude=15.4) == (
@@ -149,14 +149,14 @@ class TestGetNextRecommendation(Endpoint):
     def test_with_group(self) -> None:
         """Return recommendations for a specified group."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
-        group_id = 2  # the previous group instead
-        with all_unseen(user_id=client.user_id, group_id=group_id):
-            recommendations = Recommendation.next(user_id=client.user_id, group_id=group_id)
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
+        epoch_id = 2  # the previous group instead
+        with all_unseen(user_id=client.user_id, epoch_id=epoch_id):
+            recommendations = Recommendation.next(user_id=client.user_id, epoch_id=epoch_id)
             assert len(recommendations) == 4
-            assert all(recommendation.group_id == group_id for recommendation in recommendations)
-            assert self.get(self.route, client_id=client.id, group_id=group_id) == (
+            assert all(recommendation.epoch_id == epoch_id for recommendation in recommendations)
+            assert self.get(self.route, client_id=client.id, epoch_id=epoch_id) == (
                 STATUS['OK'], {
                     'Status': 'Success',
                     'Response': {
@@ -168,12 +168,12 @@ class TestGetNextRecommendation(Endpoint):
     def test_with_join(self) -> None:
         """Return recommendations for a specified group."""
         client = self.get_client(self.user)
-        group_id = RecommendationGroup.latest().id
-        assert group_id == 3
-        with all_unseen(user_id=client.user_id, group_id=group_id):
-            recommendations = Recommendation.next(user_id=client.user_id, group_id=group_id, limit=1)
+        epoch_id = Epoch.latest().id
+        assert epoch_id == 3
+        with all_unseen(user_id=client.user_id, epoch_id=epoch_id):
+            recommendations = Recommendation.next(user_id=client.user_id, epoch_id=epoch_id, limit=1)
             assert len(recommendations) == 1
-            assert all(recommendation.group_id == group_id for recommendation in recommendations)
+            assert all(recommendation.epoch_id == epoch_id for recommendation in recommendations)
             assert self.get(self.route, client_id=client.id, limit=1, join=True) == (
                 STATUS['OK'], {
                     'Status': 'Success',
@@ -269,7 +269,7 @@ class TestPutRecommendationAttribute(Endpoint):
 
     def test_accept(self) -> None:
         client = self.get_client(self.user)
-        with all_unseen(client.user_id, group_id=1):
+        with all_unseen(client.user_id, epoch_id=1):
             assert Recommendation.from_id(3).accepted == False
             assert self.put(self.route, client_id=client.id, accepted=True) == (
                 STATUS['OK'], {'Status': 'Success', 'Response': {}})
@@ -277,7 +277,7 @@ class TestPutRecommendationAttribute(Endpoint):
 
     def test_reject(self) -> None:
         client = self.get_client(self.user)
-        with all_unseen(client.user_id, group_id=1):
+        with all_unseen(client.user_id, epoch_id=1):
             assert Recommendation.from_id(3).rejected == False
             assert self.put(self.route, client_id=client.id, rejected=True) == (
                 STATUS['OK'], {'Status': 'Success', 'Response': {}})
@@ -346,8 +346,8 @@ class RecommendationEndpoint(Endpoint, ABC):
         )
 
 
-class TestGetGroup(RecommendationEndpoint):
-    relation = 'group'
+class TestGetEpoch(RecommendationEndpoint):
+    relation = 'epoch'
 
 
 class TestGetTag(RecommendationEndpoint):
@@ -443,35 +443,35 @@ class TestGetRecommendationHistory(Endpoint):
     user: str = 'tomb_raider'
 
     def test_invalid_parameter(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, group_id=3, foo='42') == (
+        assert self.get(self.route, client_id=self.get_client(self.user).id, epoch_id=3, foo='42') == (
             RESPONSE_MAP[ParameterInvalid], {
                 'Status': 'Error',
                 'Message': 'Unexpected parameter: foo'
             }
         )
 
-    def test_group_id_missing(self) -> None:
+    def test_epoch_id_missing(self) -> None:
         assert self.get(self.route, client_id=self.get_client(self.user).id) == (
             RESPONSE_MAP[ParameterNotFound], {
                 'Status': 'Error',
-                'Message': 'Missing expected parameter: group_id'
+                'Message': 'Missing expected parameter: epoch_id'
             }
         )
 
-    def test_group_id_not_integer(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, group_id='abc') == (
+    def test_epoch_id_not_integer(self) -> None:
+        assert self.get(self.route, client_id=self.get_client(self.user).id, epoch_id='abc') == (
             RESPONSE_MAP[ParameterNotFound], {
                 'Status': 'Error',
-                'Message': 'Expected integer for parameter: group_id'
+                'Message': 'Expected integer for parameter: epoch_id'
             }
         )
 
     def test_get_group_3(self) -> None:
-        history = Recommendation.history(user_id=3, group_id=3)
+        history = Recommendation.history(user_id=3, epoch_id=3)
         assert len(history) == 4
         assert all(recommendation.accepted for recommendation in history)
-        assert all(recommendation.group_id == 3 for recommendation in history)
-        assert self.get(self.route, client_id=self.get_client(self.user).id, group_id=3) == (
+        assert all(recommendation.epoch_id == 3 for recommendation in history)
+        assert self.get(self.route, client_id=self.get_client(self.user).id, epoch_id=3) == (
             STATUS['OK'], {
                 'Status': 'Success',
                 'Response': {'recommendation': [recommendation.to_json() for recommendation in history]},
@@ -479,11 +479,11 @@ class TestGetRecommendationHistory(Endpoint):
         )
 
     def test_get_group_2(self) -> None:
-        history = Recommendation.history(user_id=3, group_id=2)
+        history = Recommendation.history(user_id=3, epoch_id=2)
         assert len(history) == 4
         assert all(recommendation.accepted for recommendation in history)
-        assert all(recommendation.group_id == 2 for recommendation in history)
-        assert self.get(self.route, client_id=self.get_client(self.user).id, group_id=2) == (
+        assert all(recommendation.epoch_id == 2 for recommendation in history)
+        assert self.get(self.route, client_id=self.get_client(self.user).id, epoch_id=2) == (
             STATUS['OK'], {
                 'Status': 'Success',
                 'Response': {'recommendation': [recommendation.to_json() for recommendation in history]},
@@ -491,129 +491,6 @@ class TestGetRecommendationHistory(Endpoint):
         )
 
 
-class TestGetRecommendationGroupMany(Endpoint):
-    """Tests for GET /recommendation/group/ endpoint."""
-
-    route: str = '/recommendation/group'
-    method: str = 'get'
-    admin: str = 'superman'
-    user: str = 'tomb_raider'
-
-    def test_invalid_parameter(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=1, foo='42') == (
-            RESPONSE_MAP[ParameterInvalid], {
-                'Status': 'Error',
-                'Message': 'Unexpected parameter: foo'
-            }
-        )
-
-    def test_limit_missing(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id) == (
-            RESPONSE_MAP[ParameterNotFound], {
-                'Status': 'Error',
-                'Message': 'Missing expected parameter: limit'
-            }
-        )
-
-    def test_limit_too_large(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=1000) == (
-            RESPONSE_MAP[PayloadTooLarge], {
-                'Status': 'Error',
-                'Message': 'Must provide \'limit\' less than 100'
-            }
-        )
-
-    def test_limit_not_integer(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit='abc') == (
-            RESPONSE_MAP[ParameterNotFound], {
-                'Status': 'Error',
-                'Message': 'Expected integer for parameter: limit'
-            }
-        )
-
-    def test_offset_not_integer(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=4, offset='abc') == (
-            RESPONSE_MAP[ParameterNotFound], {
-                'Status': 'Error',
-                'Message': 'Expected integer for parameter: offset'
-            }
-        )
-
-    def test_get_all(self) -> None:
-        data = [group.to_json() for group in RecommendationGroup.select(20)]
-        assert len(data) == 3
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=20) == (
-            STATUS['OK'], {
-                'Status': 'Success',
-                'Response': {'recommendation_group': data},
-            }
-        )
-
-    def test_get_with_limit(self) -> None:
-        data = [group.to_json() for group in RecommendationGroup.select(20)]
-        assert len(data) == 3
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=2) == (
-            STATUS['OK'], {
-                'Status': 'Success',
-                'Response': {'recommendation_group': data[:2]},
-            }
-        )
-
-    def test_get_with_offset_1(self) -> None:
-        data = [group.to_json() for group in RecommendationGroup.select(20)]
-        assert len(data) == 3
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=2, offset=1) == (
-            STATUS['OK'], {
-                'Status': 'Success',
-                'Response': {'recommendation_group': data[1:]},
-            }
-        )
-
-    def test_get_with_offset_2(self) -> None:
-        data = [group.to_json() for group in RecommendationGroup.select(20)]
-        assert len(data) == 3
-        assert self.get(self.route, client_id=self.get_client(self.user).id, limit=2, offset=2) == (
-            STATUS['OK'], {
-                'Status': 'Success',
-                'Response': {'recommendation_group': data[2:]},
-            }
-        )
-
-
-class TestGetRecommendationGroup(Endpoint):
-    """Tests for GET /recommendation/group/<id> endpoint."""
-
-    route: str = '/recommendation/group/1'
-    method: str = 'get'
-    admin: str = 'superman'
-    user: str = 'tomb_raider'
-
-    def test_invalid_parameter(self) -> None:
-        assert self.get(self.route, client_id=self.get_client(self.user).id, foo='42') == (
-            RESPONSE_MAP[ParameterInvalid], {
-                'Status': 'Error',
-                'Message': 'Unexpected parameter: foo'
-            }
-        )
-
-    def test_not_found(self) -> None:
-        assert self.get('/recommendation/group/0', client_id=self.get_client(self.user).id) == (
-            RESPONSE_MAP[NotFound], {
-                'Status': 'Error',
-                'Message': 'No recommendation_group with id=0'
-            }
-        )
-
-    def test_get(self) -> None:
-        data = RecommendationGroup.from_id(1).to_json()
-        assert self.get(self.route, client_id=self.get_client(self.user).id) == (
-            STATUS['OK'], {
-                'Status': 'Success',
-                'Response': {'recommendation_group': data},
-            }
-        )
-
-
 @contextmanager
 def temp_remove_observation_and_file(file_id: int) -> None:
     """Remove records from database temporarily."""
